@@ -1,7 +1,6 @@
 import logging
 from typing import *
-from .scpi_transport_base import ScpiTransportBase
-from .scpi_transport_serial import ScpiSerialTransport
+from .scpi_resource import ScpiResource
 from .scpi_type_base import ScpiTypes
 from .scpi_types import ScpiEvent
 from .scpi_exceptions import ScpiErrorException
@@ -10,10 +9,10 @@ logger = logging.getLogger(__name__)
 
 
 class ScpiDevice(object):
-    def __init__(self, resource: str, transport_class: Type[ScpiTransportBase] = ScpiSerialTransport) -> None:
+    def __init__(self, resource: ScpiResource, **kwargs) -> None:
         self.resource = resource
-        self.transport = transport_class(resource)
-        logger.info(f'open [{transport_class.__name__}]: {resource}')
+        self.transport = resource.transport(resource.address)
+        logger.info(f'open [{resource.transport.transport_name}]: {resource.address}')
 
     def is_open(self):
         return self.transport.is_open
