@@ -107,6 +107,55 @@ class ScpiNumber(ScpiTypeBase):
 
         return ScpiNumber(value, unit)
 
+class ScpiNumberArray(ScpiTypeBase):
+    """
+    Class representing an array of IEEE488.2 flexible Decimal/Nondecimal numeric NRf program data (7.7.2/7.7.4) and
+    NR1/NR2/NR3 response data (8.7.2/8.7.3/8.7.4)
+    """
+
+    def __init__(self, array: Union[List[int], List[float]]) -> None:
+        """Initializes the object from a list of either an int (NR1) or float (NR2/NR3) values"""
+        super().__init__()
+        self._array = array
+
+    def __str__(self) -> str:
+        return str(self._array)
+
+    def as_int_list(self) -> List[int]:
+        """Returns the numeric data as a Python list of ints"""
+        return [int(x) for x in self._array]
+
+    def as_float_list(self) -> List[float]:
+        """Returns the numeric data as a Python list of floats"""
+        return [float(x) for x in self._array]
+
+    def compose(self) -> str:
+        """Composes the object into a SCPI string to be used as a parameter"""
+        if type(self._array[0]) == int:
+            # Compose into NR1 representation
+            return ','.join([f'{x:d}' for x in self._array])
+        elif type(self._array[0]) == float:
+            # Compose into NR2/3 representation
+            return ','.join([f'{x:e}' for x in self._array])
+        else:
+            raise TypeError
+
+    @classmethod
+    def parse(cls, transport: ScpiTransports):
+        """Parses stream data from the device and creates a `ScpiNumber` object"""
+        data = transport.readline()
+        substrs = data.split(',')
+
+        try:
+            # Try as NR1 representation first
+            array = [int(substr) for substr in substrs]
+        except ValueError:
+            try:
+                array = [float(substr) for substr in substrs]
+            except ValueError:
+                raise TypeError
+
+        return ScpiNumberArray(array)
 
 class ScpiString(ScpiTypeBase):
     """"
@@ -161,6 +210,56 @@ class ScpiString(ScpiTypeBase):
         return ScpiString(string)
 
 
+class ScpiArbBlock(ScpiTypeBase):
+    """
+    Class representing IEEE488.2 Arbitrary block program data (7.7.6) and
+    definite length arbitrary block response (8.7.9)
+
+        This element allows any 8 bit bytes to be transmitted in a message.
+        This element is particularly helpful for sending large quantities of data.
+    """
+    def __init__(self, data: bytes):
+        super().__init__()
+        self._data = data
+
+    def __str__(self) -> str:
+        return f'<block with {len(self._data)} bytes of binary data>'
+
+    def __bytes__(self) -> bytes:
+        return self.as_bytes()
+
+    def as_bytes(self) -> bytes:
+        return self._data
+
+    def compose(self) -> str:
+        length = f'{len(self._data)}'
+        head = f'{len(length)}'
+        return '#' + head + length + self._data.decode('iso-8859-15')
+
+    @classmethod
+    def parse(cls, transport: ScpiTransports):
+        """Parses stream data from the device and creates a `ScpiArbBlock` object"""
+        head = transport.read(2)
+
+        if head[0:1] != b'#':
+            raise ValueError("Could not find hash sign (#) indicating the start of the block.")
+
+        # read header size
+        num = int(head[1:2])
+
+        if num == 0:
+            raise NotImplementedError("Indefinite length arbitrary block response not implemented")
+
+        # read header
+        length = int(transport.read(num))
+        data = transport.read(length)
+
+        # Swallow \r\n
+        transport.readline()
+
+        return ScpiArbBlock(data)
+
+
 class ScpiArbStream(ScpiTypeBase, io.RawIOBase):
     """
     Class representing IEEE488.2 Arbitrary block program data (7.7.6) and
@@ -334,7 +433,7 @@ class ScpiEvent(ScpiTypeBase):
 
     def as_string(self) -> str:
         """Return the event as Python str-object"""
-        return f'{self._code:d}: {self._description}' + (' ' + self._info if self._info is not None else '')
+        return f'{self._code:d}: {self._description}' + (f' ({self._info})' if self._info is not None else '')
 
     @property
     def code(self):
